#include "mpi.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static MPI_Session lib_shandle = MPI_SESSION_NULL;
static MPI_Comm lib_comm = MPI_COMM_NULL;
static int ret = 0;

int library_foo_init(void) {
  int rc, flag, valuelen;
  const char pset_name[] = "mpi://WORLD";
  const char mt_key[] = "thread_level";
  const char mt_value[] = "MPI_THREAD_MULTIPLE";
  char out_value[100]; /* large enough */
  MPI_Group wgroup = MPI_GROUP_NULL;
  MPI_Info sinfo = MPI_INFO_NULL;
  MPI_Info tinfo = MPI_INFO_NULL;

  MPI_Info_create(&sinfo);
  MPI_Info_set(sinfo, mt_key, mt_value);
  rc = MPI_Session_init(sinfo, MPI_ERRORS_RETURN, &lib_shandle);
  if (rc != MPI_SUCCESS) {
    printf("Failed setting session info\n");
    ret = -1;
    goto fn_exit;
  }

  /*
   * check we got thread support level foo library needs
   */
  rc = MPI_Session_get_info(lib_shandle, &tinfo);
  if (rc != MPI_SUCCESS) {
    printf("Failed getting session info\n");
    ret = -1;
    goto fn_exit;
  }

  valuelen = sizeof(out_value);
  MPI_Info_get_string(tinfo, mt_key, &valuelen, out_value, &flag);
  if (0 == flag) {
    printf("Could not find key %s\n", mt_key);
    ret = -1;
    goto fn_exit;
  }

  if (strcmp(out_value, mt_value)) {
    printf("Did not get thread multiple support, got %s\n", out_value);
    ret = -1;
    goto fn_exit;
  }

  /*
   * create a group from the WORLD process set
   */
  rc = MPI_Group_from_session_pset(lib_shandle, pset_name, &wgroup);
  if (rc != MPI_SUCCESS) {
    printf("Failed getting lib group\n");
    ret = -1;
    goto fn_exit;
  }

  /*
   * get a communicator
   */
  rc = MPI_Comm_create_from_group(wgroup,
                                  "org.mpi-forum.mpi-v4_0.example-ex11_8",
                                  MPI_INFO_NULL, MPI_ERRORS_RETURN, &lib_comm);
  if (rc != MPI_SUCCESS) {
    printf("Failed getting lib comm\n");
    ret = -1;
    goto fn_exit;
  }

  int rank = -1;
  MPI_Comm_rank(lib_comm, &rank);
  printf("Hello from session rank %d\n", rank);
  /*
   * free group, library doesn't need it.
   */

  if (ret != -1)
    return MPI_SUCCESS;
fn_exit:
  printf("Closing lib\n");
  MPI_Group_free(&wgroup);

  if (sinfo != MPI_INFO_NULL) {
    MPI_Info_free(&sinfo);
  }

  if (tinfo != MPI_INFO_NULL) {
    MPI_Info_free(&tinfo);
  }

  if (ret != 0) {
    MPI_Session_finalize(&lib_shandle);
  }

  return ret;
}

int library_foo_finalize() {
  printf("lib internal ret = %d\n", ret);
  if (ret == 0) {
    printf("Closing lib\n");
    return MPI_Session_finalize(&lib_shandle);
  }
}
